//---------------------------------------------------------------------------
// File: TMainForm.cpp, TDx_3DI_Library, Landscape
//
// Date: 13th January, 2004
// Author: Darren John Dwyer
// Copyright: (c) 2002-2004 Darren John Dwyer, All Rights Reserved.
//
// Description:
// This application shows how to create and use some of the features of
// Direct3D.
//
//---------------------------------------------------------------------------
#include <vcl\vcl.h>
#pragma hdrstop

#include "TMainForm.h"
//---------------------------------------------------------------------------
#pragma link "TDx_3DI_Library_Install"
//---------------------------------------------------------------------------
#pragma link "TDx_3D"
#pragma link "TD3DMatrix"
#pragma link "TD3DViewPort"
#pragma link "TDDBltFX"
#pragma link "TDDPixelFormat"
#pragma link "TDDSCaps"
#pragma link "TDDSurfaceDesc"
#pragma link "TDx_3D"
#pragma link "TDx_3DDevice"
#pragma link "TDx_Draw"
#pragma link "TDx_DrawClipper"
#pragma link "TDx_DrawSurface"
#pragma resource "*.dfm"
//---------------------------------------------------------------------------
TMainForm *MainForm;
//---------------------------------------------------------------------------
int land_data[64][64];
D3DLVERTEX vertices[64][128];
D3DLVERTEX vertical[64][64];
D3DLVERTEX horizontal[64][64];
D3DVALUE rotationx = 0.0f;
D3DVALUE rotationy = 1.0f;
D3DVALUE vdistance = 100.0f;
//---------------------------------------------------------------------------
inline double sing(int grad)
{
   return sin((M_PI/180.0)*(double)grad);
}
//---------------------------------------------------------------------------
inline double cosg(int grad)
{
   return cos((M_PI/180.0)*(double)grad);
}
//---------------------------------------------------------------------------
VOID SetIdentityMatrix( TD3DMatrix* m )
{
    m->_12 = 0.0f;
    m->_13 = 0.0f;
    m->_14 = 0.0f;
    m->_21 = 0.0f;
    m->_23 = 0.0f;
    m->_24 = 0.0f;
    m->_31 = 0.0f;
    m->_32 = 0.0f;
    m->_34 = 0.0f;
    m->_41 = 0.0f;
    m->_42 = 0.0f;
    m->_43 = 0.0f;
    m->_11 = 0.0f;
    m->_22 = 0.0f;
    m->_33 = 0.0f;
    m->_44 = 1.0f;
}
//---------------------------------------------------------------------------
HRESULT SetViewMatrix( TD3DMatrix* mat, D3DVECTOR& vFrom,
                               D3DVECTOR& vAt, D3DVECTOR& vWorldUp )
{
    // Get the z basis vector, which points straight ahead. This is the
    // difference from the eyepoint to the lookat point.
    D3DVECTOR vView = vAt - vFrom;

    FLOAT fLength = Magnitude( vView );
    if( fLength < 1e-6f )
        return E_INVALIDARG;

    // Normalize the z basis vector
    vView /= fLength;

    // Get the dot product, and calculate the projection of the z basis
    // vector onto the up vector. The projection is the y basis vector.
    FLOAT fDotProduct = DotProduct( vWorldUp, vView );

    D3DVECTOR vUp = vWorldUp - fDotProduct * vView;

    // If this vector has near-zero length because the input specified a
    // bogus up vector, let's try a default up vector
    if( 1e-6f > ( fLength = Magnitude( vUp ) ) )
    {
        vUp = D3DVECTOR( 0.0f, 1.0f, 0.0f ) - vView.y * vView;

        // If we still have near-zero length, resort to a different axis.
        if( 1e-6f > ( fLength = Magnitude( vUp ) ) )
        {
            vUp = D3DVECTOR( 0.0f, 0.0f, 1.0f ) - vView.z * vView;

            if( 1e-6f > ( fLength = Magnitude( vUp ) ) )
                return E_INVALIDARG;
        }
    }

    // Normalize the y basis vector
    vUp /= fLength;

    // The x basis vector is found simply with the cross product of the y
    // and z basis vectors
    D3DVECTOR vRight = CrossProduct( vUp, vView );

    // Start building the matrix. The first three rows contains the basis
    // vectors used to rotate the view to point at the lookat point
    SetIdentityMatrix( mat );
    mat->_11 = vRight.x;    mat->_12 = vUp.x;    mat->_13 = vView.x;
    mat->_21 = vRight.y;    mat->_22 = vUp.y;    mat->_23 = vView.y;
    mat->_31 = vRight.z;    mat->_32 = vUp.z;    mat->_33 = vView.z;

    // Do the translation values (rotations are still about the eyepoint)
    mat->_41 = - DotProduct( vFrom, vRight );
    mat->_42 = - DotProduct( vFrom, vUp );
    mat->_43 = - DotProduct( vFrom, vView );

    return S_OK;
}
//---------------------------------------------------------------------------
HRESULT SetProjectionMatrix( TD3DMatrix* matrix, FLOAT fFOV, FLOAT fAspect,
                                     FLOAT fNearPlane, FLOAT fFarPlane )
{
    if( fabs(fFarPlane-fNearPlane) < 0.01f )
        return E_INVALIDARG;

    if( fabs(sin(fFOV/2)) < 0.01f )
        return E_INVALIDARG;

    FLOAT w = fAspect * ( cos(fFOV/2)/sin(fFOV/2) );
    FLOAT h =   1.0f  * ( cos(fFOV/2)/sin(fFOV/2) );
    FLOAT Q = fFarPlane / ( fFarPlane - fNearPlane );

    matrix->Clear();
    matrix->_11 = w;
    matrix->_22 = h;
    matrix->_33 = Q;
    matrix->_34 = 1.0f;
    matrix->_43 = -Q*fNearPlane;
}
//---------------------------------------------------------------------------
// Diamond-Square Algorithmus
//---------------------------------------------------------------------------
void diasq(int x1,int y1,int x2,int y2,int spanne)
{
  int xm,ym,zm;
  if (abs(x2-x1)>1)
  {
    xm=(int)((x1+x2) / 2);
    ym=(int)((y1+y2) / 2);
    zm=(int)((land_data[x1][y1]+land_data[x2][y1]+land_data[x1][y2]+land_data[x2][y2]) / 4);
    if (land_data[xm][ym]==0)
      { land_data[xm][ym]=(int)(zm+random(spanne));}
    if (land_data[xm][y1]==0)
      { land_data[xm][y1]=(int)((land_data[x1][y1]+land_data[x2][y1]) / 2 + random(spanne)); }
    if (land_data[xm][y2]==0)
      { land_data[xm][y2]=(int)((land_data[x2][y2]+land_data[x1][y2]) / 2 + random(spanne)); }
    if (land_data[x1][ym]==0)
      { land_data[x1][ym]=(int)((land_data[x1][y1]+land_data[x1][y2]) / 2 + random(spanne)); }
    if (land_data[x2][ym]==0)
      { land_data[x2][ym]=(int)((land_data[x2][y2]+land_data[x2][y1]) / 2 + random(spanne)); }
    diasq(x1,y1,xm,ym,(int)(spanne / 2));
    diasq(xm,y1,x2,ym,(int)(spanne / 2));
    diasq(x1,ym,xm,y2,(int)(spanne / 2));
    diasq(xm,ym,x2,y2,(int)(spanne / 2));
  }
}
//---------------------------------------------------------------------------
DWORD LandColor(int height)
{
   if (height<14) return RGB(255,height*10,height*10);
   if (height<24) return RGB(0,height*6,0);
   return RGB(200,200,200);
}
//---------------------------------------------------------------------------
void BuildLandscape()
{
diasq(0,0,64,64,24);
for (int x=0; x<64; x++)
    for (int y=0; y<64; y++)
    	{
       	vertical[x][y]=D3DLVERTEX(D3DVECTOR(x-32,land_data[x][y]-10,y-32),0x00ff00,0xFF,0,0);
       	horizontal[y][x]=D3DLVERTEX(D3DVECTOR(x-32,land_data[x][y]-10,y-32),0x00ff00,0xFF,0,0);
    	}
 for (int x=0; x<63; x++)
 	for (int y=0; y<63; y++)
     	{
        vertices[x][2*y  ]=D3DLVERTEX(D3DVECTOR(x-32,(land_data[  x][y]<12)? 4:land_data[  x][y]-10,y-32),LandColor(land_data[  x][y]),0xFF,0,0);
        vertices[x][2*y+1]=D3DLVERTEX(D3DVECTOR(x-31,(land_data[1+x][y]<12)? 4:land_data[1+x][y]-10,y-32),LandColor(land_data[1+x][y]),0xFF,0,0);
     	}
}
//---------------------------------------------------------------------------
__fastcall TMainForm::TMainForm(TComponent* Owner)
	: TForm(Owner)
{
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::StartButtonClick(TObject *Sender)
{
// disable start button

StartButton->Enabled = false;

// create DirectDraw interface

if (DirectDraw->Create( NULL ))
	{
    DirectDraw_Label->Caption = "Created";
    DirectDraw_Label->Update();
    }

// choose non-fullscreen or full-screen/exclusive

if (!FullScreenCheckBox->Checked)
	{
    // non-fullscreen DirectDraw setup

	if (DirectDraw->SetCooperativeLevel( Handle, DDSCL_NORMAL ))
    	{
    	CooperativeLevel_Label->Caption = "Normal";
    	CooperativeLevel_Label->Update();
        }

    // create DirectDraw primary surface

    PrimarySurfaceDesc->Flags = 0;
    PrimarySurfaceDesc->SCaps = PrimarySurfaceCaps;
    PrimarySurfaceDesc->SCaps->Caps = DDSCAPS_PRIMARYSURFACE;

	if (PrimarySurface->Create( PrimarySurfaceDesc, DirectDraw ))
    	{
        PrimarySurface_Label->Caption = "Created";
        PrimarySurface_Label->Update();
        }

    // create / retrieve backbuffer

    PrimarySurfaceDesc->Flags = DDSD_WIDTH | DDSD_HEIGHT;
    PrimarySurfaceDesc->SCaps = PrimarySurfaceCaps;
    PrimarySurfaceDesc->SCaps->Caps = DDSCAPS_OFFSCREENPLAIN | DDSCAPS_3DDEVICE;
    PrimarySurfaceDesc->Width = ClientWidth;
    PrimarySurfaceDesc->Height = ClientHeight;

    if (BackBuffer->Create( PrimarySurfaceDesc, DirectDraw ))
    	{
        BackBuffer_Label->Caption = "Created";
        BackBuffer_Label->Update();
        }
	}
else
	{
    // fullscreen complex setup

    if (DirectDraw->SetCooperativeLevel( Handle, DDSCL_FULLSCREEN | DDSCL_EXCLUSIVE ))
    	{
        CooperativeLevel_Label->Caption = "Fullscreen / Exclusive";
        CooperativeLevel_Label->Update();
        }

    // create DirectDraw primary surface

    PrimarySurfaceDesc->Flags = DDSD_CAPS | DDSD_BACKBUFFERCOUNT;
    PrimarySurfaceDesc->SCaps = PrimarySurfaceCaps;
    PrimarySurfaceDesc->SCaps->Caps = DDSCAPS_PRIMARYSURFACE | DDSCAPS_FLIP | DDSCAPS_3DDEVICE | DDSCAPS_COMPLEX;
    PrimarySurfaceDesc->BackBufferCount = 1;

    if (PrimarySurface->Create( PrimarySurfaceDesc, DirectDraw ))
    	{
        PrimarySurface_Label->Caption = "Created";
        PrimarySurface_Label->Update();
        }

    // create / retrieve backbuffer

    PrimarySurfaceCaps->Clear();
    PrimarySurfaceCaps->Caps = DDSCAPS_BACKBUFFER;

    if (PrimarySurface->GetAttachedSurface( PrimarySurfaceCaps, BackBuffer ))
    	{
        BackBuffer_Label->Caption = "Created";
        BackBuffer_Label->Update();
        }
    }

// create a clipper for the primary surface

if (PrimaryClipper->Create( DirectDraw ))
    {
    PrimaryClipper_Label->Caption = "Created";
    PrimaryClipper_Label->Update();
    }

PrimaryClipper->SetHWnd( Handle );
if (PrimarySurface->SetClipper( PrimaryClipper ))
    {
    PrimaryClipper_Label->Caption = PrimaryClipper_Label->Caption + ", Attached to Primary Surface";
    PrimaryClipper_Label->Update();
    }

// create d3d interface

if (Direct3DI->Create( DirectDraw ))
	{
    Direct3DI_Label->Caption = "Created";
    Direct3DI_Label->Update();
    }

// create hardware device

if (Direct3DI->CreateDevice( IID_IDirect3DHALDevice, BackBuffer, Direct3DIDevice ))
	{
    Direct3DIDevice_Label->Caption = "Created";
    Direct3DIDevice_Label->Update();
    }

// create the viewport

ViewPort3D->Clear();
ViewPort3D->X = 0;
ViewPort3D->Y = 0;
ViewPort3D->Width = ClientWidth;
ViewPort3D->Height = ClientHeight;
ViewPort3D->MinZ = 0.0;
ViewPort3D->MaxZ = 1.0;

if (Direct3DIDevice->SetViewPort( ViewPort3D ))
	{
    ViewPort3D_Label->Caption = "Assigned to Direct3D Device";
    ViewPort3D_Label->Update();
    }

// do a quick projection

SetProjectionMatrix( D3DMatrix1, 0.785f, 1.333f, 10.0, 1000.0);

Direct3DIDevice->SetTransform( D3DTRANSFORMSTATE_PROJECTION, D3DMatrix1 );
Direct3DIDevice->SetRenderState( D3DRENDERSTATE_LIGHTING, FALSE );
Direct3DIDevice->SetRenderState( D3DRENDERSTATE_ZENABLE, TRUE );
Direct3DIDevice->SetRenderState( D3DRENDERSTATE_CULLMODE, D3DCULL_NONE );
//Direct3DIDevice->SetRenderState( D3DRENDERSTATE_CULLMODE, D3DCULL_CW );
//Direct3DIDevice->SetRenderState( D3DRENDERSTATE_CULLMODE, D3DCULL_CCW );
//Direct3DIDevice->SetRenderState( D3DRENDERSTATE_CULLMODE, D3DCULL_FORCE_DWORD );

// create a landcape

randomize();
for (int x=0; x<64; x++)
   for (int y=0; y<64; y++)
       land_data[x][y]=0;

BuildLandscape();

// begin animation

Timer1->Enabled = true;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ExitButtonClick(TObject *Sender)
{
Timer1->Enabled = false;

// not actually needed as the components auto-cleanup on destroy
Texture3D->Destroy();
Direct3DIDevice->Destroy();
Direct3DI->Destroy();
PrimaryClipper->Destroy();
BackBuffer->Destroy();
PrimarySurface->Destroy();
DirectDraw->FlipToGDISurface();
DirectDraw->Destroy();

Close();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::DirectDrawError(TObject *Sender, AnsiString Function,
	AnsiString Error, AnsiString ErrorMessage)
{
ErrorMemo->Lines->Add( "-----" );
ErrorMemo->Lines->Add( "Error" );
ErrorMemo->Lines->Add( "-----" );
ErrorMemo->Lines->Add( "Function: "+Function );
ErrorMemo->Lines->Add( "Error: "+Error );
ErrorMemo->Lines->Add( "ErrorMessage: "+ErrorMessage );
ErrorMemo->Lines->Add( "-----" );
}
//---------------------------------------------------------------------------7
void __fastcall TMainForm::Timer1Timer(TObject *Sender)
{
Animating_Label->Caption = "Animating";
Animating_Label->Update();

Application->ProcessMessages();
RenderScene(); // paint to the backbuffer
DisplayScene(); // flip to primary surface
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::RenderScene()
{
// clear the backbuffer black
TRect r = BoundsRect;
DDBltFX1->FillColor = ColorToRGB( clBlack );
BackBuffer->Blt( NULL, NULL, NULL, DDBLT_COLORFILL, DDBltFX1 );

// display 3D over the top
SetViewMatrix( D3DMatrix1, D3DVECTOR( cos(rotationx)*cos(rotationy)*vdistance,
									  sin(rotationy)*vdistance,
                                      sin(rotationx)*cos(rotationy)*vdistance),
                           D3DVECTOR(0,0,0),
                           D3DVECTOR(0,0,0) );

Direct3DIDevice->SetTransform( D3DTRANSFORMSTATE_VIEW, D3DMatrix1 );
Direct3DIDevice->Clear( 0,0, D3DCLEAR_ZBUFFER | D3DCLEAR_TARGET, 0, 1.0, 0 );

if (Direct3DIDevice->BeginScene())
	{
    BeginScene_Label->Caption = "working";
    BeginScene_Label->Update();
    }

for (int i=0; i<63; i++)
	if (Direct3DIDevice->DrawPrimitive(D3DPT_TRIANGLESTRIP,D3DFVF_LVERTEX,vertices[i],126,0))
    	{
        DrawPrimitive_Label->Caption = "working";
        DrawPrimitive_Label->Update();
        }

if (ShowLinesCheckBox->Checked)
	{
    bool result = true;
    for (int i=0; i<64; i++)
        {
        result &= Direct3DIDevice->DrawPrimitive(D3DPT_LINESTRIP,D3DFVF_LVERTEX,vertical[i],64,0);
        result &= Direct3DIDevice->DrawPrimitive(D3DPT_LINESTRIP,D3DFVF_LVERTEX,horizontal[i],64,0);
        }
    if (result)
        {
        DrawPrimitive_Label->Caption = "working";
        DrawPrimitive_Label->Update();
        }
    else
    	{
        DrawPrimitive_Label->Caption = "not working";
        DrawPrimitive_Label->Update();
        }
    }

if (Direct3DIDevice->EndScene())
	{
    EndScene_Label->Caption = "working";
    EndScene_Label->Update();
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::DisplayScene()
{
// done.
TRect r = BoundsRect;
PrimarySurface->Flip( BackBuffer, DDFLIP_WAIT );
DirectDraw->WaitForVerticalBlank( DDWAITVB_BLOCKBEGIN );
PrimarySurface->Blt( &r, BackBuffer, NULL, DDBLT_WAIT, NULL );

if (PrimarySurface->ErrorValue == DDERR_SURFACELOST)
	DirectDraw->RestoreAllSurfaces();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ShowStuffCheckBoxClick(TObject *Sender)
{
if (ShowStuffCheckBox->Checked)
	{
    Stuff->Visible = true;
    ErrorMessages->Visible = true;
    }
else
	{
    Stuff->Visible = false;
    ErrorMessages->Visible = false;
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Direct3DIEnumZBufferFormats(TObject *Sender,
	TDDPixelFormat *DeviceDesc, void *Context, bool &Continue)
{
if (DeviceDesc->Flags[0] & DDPF_ZBUFFER)
	{
    DDPixelFormat1->ArraySize = 1;
    CopyMemory( DDPixelFormat1->Internal_DDPIXELFORMAT_Ptr,
    			DeviceDesc->Internal_DDPIXELFORMAT_Ptr,
                sizeof( DDPIXELFORMAT ) );
    Continue = false;
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::CenterButtonClick(TObject *Sender)
{
rotationx = 0;
rotationy = 0;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::UpButtonMouseDown(TObject *Sender,
	TMouseButton Button, TShiftState Shift, int X, int Y)
{
rotationy-=0.05;
if (rotationy<0) rotationy=6.283f;
Timer1Timer(this);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::LtButtonMouseDown(TObject *Sender,
	TMouseButton Button, TShiftState Shift, int X, int Y)
{
rotationx-=0.05;
if (rotationx<0) rotationx=6.283f;
Timer1Timer(this);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::RtButtonMouseDown(TObject *Sender,
	TMouseButton Button, TShiftState Shift, int X, int Y)
{
rotationx+=0.05;
if (rotationx>6.283f) rotationx=0;
Timer1Timer(this);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::DnButtonMouseDown(TObject *Sender,
	TMouseButton Button, TShiftState Shift, int X, int Y)
{
rotationy+=0.05;
if (rotationy>6.283f) rotationy=0;
Timer1Timer(this);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormCreate(TObject *Sender)
{
Top=0;
Left=0;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormKeyDown(TObject *Sender, WORD &Key,
	TShiftState Shift)
{
if (Key==VK_LEFT) LtButtonMouseDown( this, TMouseButton(), TShiftState(), 0, 0 );
if (Key==VK_RIGHT) RtButtonMouseDown( this, TMouseButton(), TShiftState(), 0, 0 );
if (Key==VK_UP) UpButtonMouseDown( this, TMouseButton(), TShiftState(), 0, 0 );
if (Key==VK_DOWN) DnButtonMouseDown( this, TMouseButton(), TShiftState(), 0, 0 );
if (Key==VK_HOME) ZoomInButtonMouseDown( this, TMouseButton(), TShiftState(), 0, 0 );
if (Key==VK_END) ZoomOutButtonMouseDown( this, TMouseButton(), TShiftState(), 0, 0 );
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ZoomInButtonMouseDown(TObject *Sender,
	TMouseButton Button, TShiftState Shift, int X, int Y)
{
vdistance -= 1.0;
Timer1Timer(this);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ZoomOutButtonMouseDown(TObject *Sender,
	TMouseButton Button, TShiftState Shift, int X, int Y)
{
vdistance += 1.0;
Timer1Timer(this);
}
//---------------------------------------------------------------------------
